@@ -4,6 +4,7 @@
 
 import collections
 import datetime
+import errno
 import functools
 import logging
 import os
@@ -12,6 +13,7 @@ import struct
 import sys
 import time
 import threading
+import warnings
 from zipfile import (
     # Classes
     ZipInfo,
@@ -59,6 +61,8 @@ __all__ = [
     "ZipStream", "ZipStreamInfo",
     # Compression constants (imported from zipfile)
     "ZIP_STORED", "ZIP_DEFLATED", "BZIP2_VERSION", "ZIP_BZIP2", "LZMA_VERSION", "ZIP_LZMA",
+    # Helper functions
+    "walk"
 ]
 
 __log__ = logging.getLogger(__name__)
@@ -122,7 +126,7 @@ class ZipStreamInfo(ZipInfo):
         # streamable
         return struct.pack(
             "<4sLQQ" if zip64 else "<4sLLL",
-            b'PK\x07\x08',  # Data descriptor signature
+            b"PK\x07\x08",  # Data descriptor signature
             self.CRC,
             self.compress_size,
             self.file_size
@@ -261,15 +265,19 @@ class ZipStreamInfo(ZipInfo):
         self.file_size = file_size
         self.compress_size = compress_size
 
-        if not zip64 and max(file_size, compress_size) > ZIP64_LIMIT:  # pragma: no cover
+        if not zip64 and max(file_size, compress_size) > ZIP64_LIMIT:
             # Didn't estimate correctly :(
-            raise RuntimeError("Adding file unexpectedly required using Zip64")
+            raise RuntimeError(
+                "Adding file '{}' unexpectedly required using Zip64 extensions".format(
+                    self.filename
+                )
+            )
 
         # Yield the data descriptor with the now-valid CRC and file size info
         yield self.DataDescriptor(zip64)
 
     def _central_directory_header_data(self):
-        """Yield a central directory file header for this file"""
+        """Return a central directory file header for this file"""
         # Based on code in zipfile.ZipFile._write_end_record
 
         dosdate, dostime = _timestamp_to_dos(self.date_time)
@@ -301,7 +309,7 @@ class ZipStreamInfo(ZipInfo):
             # ZipStreamInfo records, there shouldn't ever be any so we don't
             # bother.
             extra_data = struct.pack(
-                '<HH' + 'Q'*len(extra), 1, 8*len(extra), *extra
+                "<HH" + "Q"*len(extra), 1, 8*len(extra), *extra
             ) + extra_data
             min_version = ZIP64_VERSION
 
@@ -335,10 +343,7 @@ class ZipStreamInfo(ZipInfo):
             self.external_attr,
             header_offset
         )
-        yield centdir
-        yield filename
-        yield extra_data
-        yield self.comment
+        return centdir + filename + extra_data + self.comment
 
     if PY35_COMPAT:  # pragma: no cover
         # Backport essential functions introduced in 3.6
@@ -412,9 +417,33 @@ def _validate_compression(func):
     return wrapper
 
 
+def _sanitize_arcname(arcname):
+    """Terminate the arcname at the first null byte"""
+    # based on zipfile._sanitize_filename
+
+    if arcname:
+        # trim the arcname to the first null byte
+        null_byte = arcname.find(chr(0))
+        if null_byte >= 0:
+            arcname = arcname[:null_byte]
+
+    if not arcname:
+        raise ValueError(
+            "A valid arcname (name of the entry in the zip file) is required"
+        )
+
+    # Ensure paths in the zip always use forward slashes as the directory
+    # separator
+    for sep in PATH_SEPARATORS:
+        if sep != "/":
+            arcname = arcname.replace(sep, "/")
+
+    return arcname
+
+
 def _iter_file(path):
     """Yield data from a file"""
-    with open(path, 'rb') as fp:
+    with open(path, "rb") as fp:
         while True:
             buf = fp.read(READ_BUFFER)
             if not buf:
@@ -422,6 +451,47 @@ def _iter_file(path):
             yield buf
 
 
+def walk(path, preserve_empty=True, followlinks=True):
+    """Recursively walk the given the path and yield files/folders under it.
+
+    preserve_empty:
+        If True (the default), empty directories will be included in the
+        output. The paths of these directories will be yielded with a trailing
+        path separator.
+
+    followlinks:
+        If True (the default), symlinks to folders will be resolved and
+        followed unless this would result in infinite recursion (symlinks to
+        files are always resolved)
+    """
+
+    # Define a function to return the device and inode for a path.
+    # Will be used to deduplicate folders to avoid infinite recursion
+    def _getkey(path):
+        st = os.stat(path)
+        return (st.st_dev, st.st_ino)
+
+    visited = {_getkey(path)}
+    for dirpath, dirnames, files in os.walk(path, followlinks=followlinks):
+
+        if followlinks:
+            # Prevent infinite recursion by removing previously-visited
+            # directories from dirnames.
+            for i in reversed(range(len(dirnames))):
+                k = _getkey(os.path.join(dirpath, dirnames[i]))
+                if k in visited:
+                    dirnames.pop(i)
+                else:
+                    visited.add(k)
+
+        # Preserve empty directories
+        if preserve_empty and not files and not dirnames:
+            files = [""]
+
+        for f in files:
+            yield os.path.join(dirpath, f)
+
+
 class ZipStream:
     """A write-only zip that is generated from source files/data as it's
     iterated over.
@@ -482,7 +552,6 @@ class ZipStream:
 
         # For calculating the size
         self._sized = sized
-        self._to_count = collections.deque()
         self._size_prog = (0, 0, 0)
         self._size_lock = threading.Lock()
 
@@ -507,11 +576,27 @@ class ZipStream:
         if not self._sized:
             raise TypeError("The length of this ZipStream is unknown")
 
-        return self._get_size()
+        with self._size_lock:
+            (num_files, files_size, cdfh_size) = self._size_prog
+
+        # Calculate the amount of data the end of central directory needs. This
+        # is computed every time since it depends on the other metrics. Also,
+        # it means that we don't have to deal with detecting if the comment
+        # changes.
+        eocd_size = sizeEndCentDir + len(self._comment)  # 22 + comment len
+        if (
+            num_files > ZIP_FILECOUNT_LIMIT or
+            files_size > ZIP64_LIMIT or
+            cdfh_size > ZIP64_LIMIT
+        ):
+            eocd_size += sizeEndCentDir64  # 56
+            eocd_size += sizeEndCentDir64Locator  # 20
+
+        return cdfh_size + files_size + eocd_size
 
     def __bytes__(self):
         """Get the bytes of the ZipStream"""
-        return b''.join(self)
+        return b"".join(self)
 
     def file(self):
         """Generate data for a single file being added to the ZipStream
@@ -566,39 +651,51 @@ class ZipStream:
     def add_path(self, path, arcname=None, *, recurse=True, compress_type=None, compress_level=None):
         """Queue up a path to be added to the ZipStream
 
-        Queues the `path` up to to be written to the archive, giving it the name
-        provided by `arcname`. If `arcname` is not provided, it is assumed to be
-        the last component of the `path` (Ex: "/path/to/files/" --> "files")
+        Queues the `path` up to to be written to the archive, giving it the
+        name provided by `arcname`. If `arcname` is not provided, it is assumed
+        to be the last component of the `path` (Ex: "/path/to/files/" -->
+        "files").
 
-        If `recurse` is `True` (the default), and the `path` is a directory, all
-        files and folders under the `path` will be added to ZipStream. Symlinks
-        to files and folders will be resolved and followed unless this would
-        result in infinite recursion.
+        if `recurse` is `True` (the default), and the `path` is a directory,
+        all contents under the `path` will also be added. By default, this is
+        done using the `walk` function in this module, which will preserve
+        empty directories as well as follow symlinks to files and folders
+        unless this would result in infinite recursion.
+
+        If more control over directory walking is required, a function that
+        takes a `path` and returns an iterable of paths can also be passed in
+        as `recurse`. Alternatively, the directory can be walked in external
+        code while calling `add_path(path, arcname, recurse=False)` for each
+        discovered entry.
 
         If recurse is `False`, only the specified path (file or directory) will
         be added.
 
-        If more control over directory walking is required, walk the directory
-        normally and call `add_path(path, arcname, recurse=False)` for each
-        discovered path.
+        If given, `compress_type` and `compress_level` override the settings
+        the ZipStream was initialized with.
 
-        If given, `compress_type` and `compress_level` override the settings the
-        ZipStream was initialized with.
-
-        Raises a ValueError if the path does not exist.
+        Raises a FileNotFoundError if the path does not exist
+        Raises a ValueError if an arcname isn't provided and the assumed
+        one is empty.
         Raises a RuntimeError if the ZipStream has already been finalized.
         """
         # Resolve path objects to strings on Python 3.5
         if PY35_COMPAT and hasattr(path, "__fspath__"):  # pragma no cover
             path = path.__fspath__()
 
+        if not os.path.exists(path):
+            raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), path)
+
         path = os.path.normpath(path)
 
+        # special case - discover the arcname from the path
         if not arcname:
             arcname = os.path.basename(path)
-
-        if not os.path.exists(path):
-            raise ValueError("Path '{}' not found".format(path))
+            if not arcname:
+                raise ValueError(
+                    "No arcname for path '{}' could be assumed".format(path)
+                )
+        arcname = _sanitize_arcname(arcname)
 
         # Not recursing - just add the path
         if not recurse or not os.path.isdir(path):
@@ -610,42 +707,18 @@ class ZipStream:
             )
             return
 
-        # Define a function to return the device and inode for a path.
-        # Will be used to deduplicate folders to avoid infinite recursion
-        def _getkey(path):
-            s = os.stat(path)
-            return (s.st_dev, s.st_ino)
+        if recurse is True:
+            recurse = walk
 
-        visited = {_getkey(path)}
-        for dirpath, dirnames, files in os.walk(path, followlinks=True):
-
-            # Prevent infinite recursion by removing previously-visited
-            # directories from dirnames.
-            for i in reversed(range(len(dirnames))):
-                k = _getkey(os.path.join(dirpath, dirnames[i]))
-                if k in visited:
-                    dirnames.pop(i)
-                else:
-                    visited.add(k)
-
-            # Preserve empty directories
-            if not files and not dirnames:
-                files = [""]
-
-            for f in files:
-                filepath = os.path.join(dirpath, f)
-                filename = os.path.relpath(filepath, path)
-                filearcname = os.path.normpath(os.path.join(arcname, filename))
-                if not f:
-                    # adding an empty directory - make sure it has a trailing slash
-                    filearcname += "/"
-
-                self._enqueue(
-                    path=filepath,
-                    arcname=filearcname,
-                    compress_type=compress_type,
-                    compress_level=compress_level
-                )
+        for filepath in recurse(path):
+            filename = os.path.relpath(filepath, path)
+            filearcname = os.path.normpath(os.path.join(arcname, filename))
+            self._enqueue(
+                path=filepath,
+                arcname=filearcname,
+                compress_type=compress_type,
+                compress_level=compress_level
+            )
 
     @_validate_final
     @_validate_compression
@@ -664,8 +737,7 @@ class ZipStream:
         would create a directory (which can't contain content).
 
         `size` (optional) specifies the size of the `data` ONLY in the case
-        where it is an iterator and the ZipStream is sized. It is ignored in
-        all other cases.
+        where it is an iterator. It is ignored in all other cases.
 
         Note that the data provided will not be used until the file is actually
         encoded in the ZipStream. This means that strings and bytes will be held
@@ -676,15 +748,14 @@ class ZipStream:
         ZipStream was initialized with.
 
         Raises a ValueError if an arcname is not provided or ends with a "/"
-        when data id given.
+        when data is given.
         Raises a TypeError if the data is not str, bytes, or an iterator.
         Raises a RuntimeError if the ZipStream has already been finalized.
         """
-        if not arcname:
-            raise ValueError("A filename to store the data in is required")
+        arcname = _sanitize_arcname(arcname)
 
         if data is None:
-            data = b''
+            data = b""
         elif isinstance(data, str):
             data = data.encode("utf-8")
         elif isinstance(data, bytearray):
@@ -692,10 +763,12 @@ class ZipStream:
             # change while we're iterating over it.
             data = bytes(data)
 
-        if data != b'' and arcname[-1] in PATH_SEPARATORS:
-            raise ValueError("Can't store data as a directory")
+        is_directory = arcname[-1] in PATH_SEPARATORS
 
         if isinstance(data, bytes):
+            if is_directory and data:
+                raise ValueError("Can't store data as a directory")
+
             self._enqueue(
                 data=data,
                 arcname=arcname,
@@ -703,9 +776,12 @@ class ZipStream:
                 compress_level=compress_level,
             )
         elif hasattr(data, "__iter__"):
+            if is_directory:
+                raise ValueError("Can't store an iterable as a directory")
+
             self._enqueue(
                 iterable=data,
-                size=size if self._sized else None,
+                size=size,
                 arcname=arcname,
                 compress_type=compress_type,
                 compress_level=compress_level,
@@ -715,6 +791,15 @@ class ZipStream:
                 "Data to add must be str, bytes, or an iterable of bytes"
             )
 
+    def mkdir(self, arcname):
+        """Create a directory inside the ZipStream"""
+        arcname = _sanitize_arcname(arcname)
+
+        if arcname[-1] != "/":
+            arcname += "/"
+
+        self.add(data=None, arcname=arcname)
+
     @property
     def sized(self):
         """True if the ZipStream's final size is known"""
@@ -745,7 +830,7 @@ class ZipStream:
         Raises a RuntimeError if the ZipStream has already been finalized.
         """
         if comment is None:
-            comment = b''
+            comment = b""
         elif isinstance(comment, str):
             comment = comment.encode("utf-8")
         elif isinstance(comment, bytearray):
@@ -775,6 +860,59 @@ class ZipStream:
         """The number of files that have already been added to the stream"""
         return len(self._filelist)
 
+    def info_list(self):
+        """Get a list of dicts containing data about each file in the ZipStream
+
+        File information will be yielded in the order that the files were
+        added.
+
+        All files will be included in this list. The "streamed" key indicates
+        if the file has been written to the ZipStream or not. Files that
+        haven't yet been written to the ZipStream will be missing information
+        that's only known post-write (compressed size, CRC, datetime, etc.)
+        """
+        # Need to prevent another thread from popping a result from
+        # self._queue, then this function being run before it can be added to
+        # self._filelist
+        with self._gen_lock:
+            info = [
+                {
+                    "name": x.filename,
+                    "size": x.file_size,
+                    "compressed_size": x.compress_size,
+                    "datetime": x.date_time,
+                    "is_dir": x.is_dir(),
+                    "CRC": x.CRC,
+                    "compress_type": x.compress_type,
+                    "compress_level": getattr(x, "_compresslevel", None),  # <3.7 compat
+                    "streamed": True,
+                }
+                for x in self._filelist
+            ]
+            for x in self._queue:
+                is_dir = x["arcname"][-1] == "/"
+                compress_type = x.get("compress_type", self._compress_type)
+                if is_dir:
+                    compress_size = 0
+                elif compress_type == ZIP_STORED:
+                    compress_size = x.get("size")
+                else:
+                    compress_size = None
+
+                info.append({
+                    "name": x["arcname"],
+                    "size": x.get("size"),
+                    "compressed_size": compress_size,
+                    "datetime": None,
+                    "is_dir": is_dir,
+                    "CRC": None,
+                    "compress_type": compress_type,
+                    "compress_level": x.get("compress_level", self._compress_level),
+                    "streamed": False,
+                })
+
+        return info
+
     def get_info(self):
         """Get a list of dicts containing data about each file currently in the
         ZipStream.
@@ -782,6 +920,11 @@ class ZipStream:
         Note that this ONLY includes files that have already been written to the
         ZipStream. Queued files are NOT included.
         """
+        warnings.warn(
+            "ZipStream.get_info is deprecated and will be removed in a future "
+            "version. Use ZipStream.info_list instead",
+            DeprecationWarning,
+        )
         return [
             {
                 "name": x.filename,
@@ -827,24 +970,39 @@ class ZipStream:
         data = kwargs.get("data")
         size = kwargs.get("size")
 
+        if path:
+            st = os.stat(path)
+        else:
+            st = None
+
         # Get the modified time of the added path (use current time for
         # non-paths) and use it to update the last_modified property
-        mtime = time.localtime(os.stat(path).st_mtime if path else None)[0:6]
+        mtime = time.localtime(st.st_mtime if path else None)[0:6]
         if self._last_modified is None or self._last_modified < mtime:
             self._last_modified = mtime
 
-        # Get the expected size of the data where not specified and possible
-        if size is None:
-            if data is not None:
-                kwargs["size"] = len(data)
-            elif path is not None:
-                if os.path.isdir(path):
+        # Get the expected size of the data where possible
+        if path or size is None:
+            if path:
+                if stat.S_ISDIR(st.st_mode):
                     kwargs["size"] = 0
                 else:
-                    kwargs["size"] = os.path.getsize(path)
+                    kwargs["size"] = st.st_size
+            elif data is not None:
+                kwargs["size"] = len(data)
 
-        # If the ZipStream is sized then it will look at what is being added and
-        # queue up some information for _get_size to use to compute the total
+        # If adding data from a path ensure that the name matches the type of
+        # data being added.
+        if path:
+            name_is_dir = kwargs["arcname"][-1] == "/"
+            path_is_dir = stat.S_ISDIR(st.st_mode)
+            if path_is_dir and not name_is_dir:
+                kwargs["arcname"] += "/"
+            elif not path_is_dir and name_is_dir:
+                kwargs["arcname"] = kwargs["arcname"].rstrip("/")
+
+        # If the ZipStream is sized then it will look at what is being added
+        # and add the number of bytes used by this file to the running total
         # length of the stream. It will also read any iterables fully into
         # memory so their size is known.
         if self._sized:
@@ -853,11 +1011,17 @@ class ZipStream:
 
             # Iterate the iterable data to get the size and replace it with the static data
             if path is None and data is None and size is None:
-                data = b''.join(kwargs.pop("iterable"))
+                data = b"".join(kwargs.pop("iterable"))
                 kwargs["size"] = len(data)
                 kwargs["data"] = data
 
-            self._to_count.append((kwargs["arcname"], kwargs["size"]))
+            self._add_size_from_file(kwargs["arcname"], kwargs["size"])
+
+        # Remove any default/redundant compression parameters
+        if kwargs.get("compress_type") in (None, self._compress_type):
+            kwargs.pop("compress_type", None)
+        if kwargs.get("compress_level") in (None, self._compress_level):
+            kwargs.pop("compress_level", None)
 
         self._queue.append(kwargs)
 
@@ -866,7 +1030,7 @@ class ZipStream:
         self._pos += len(data)
         return data
 
-    def _gen_file_entry(self, *, path=None, iterable=None, data=None, size=None, arcname, compress_type, compress_level):
+    def _gen_file_entry(self, *, path=None, iterable=None, data=None, size=None, arcname, compress_type=None, compress_level=None):
         """Yield the zipped data generated by the specified path/iterator/data"""
         assert bool(path) ^ bool(iterable) ^ bool(data is not None)
         assert not (self._sized and size is None)
@@ -884,17 +1048,23 @@ class ZipStream:
 
             if data is not None:
                 zinfo.file_size = len(data)
+            elif size is not None:
+                zinfo.file_size = size
 
         zinfo.compress_type = compress_type if compress_type is not None else self._compress_type
         if not PY36_COMPAT:
-            zinfo._compresslevel = compress_level if compress_level is not None else self._compress_level
+            if zinfo.compress_type in (ZIP_STORED, ZIP_LZMA):
+                # Make sure the zinfo properties are accurate for info_list
+                zinfo._compresslevel = None
+            else:
+                zinfo._compresslevel = compress_level if compress_level is not None else self._compress_level
 
         # Store the position of the header
         zinfo.header_offset = self._pos
 
-        # We need to force using zip64 extensions for iterables since we don't
-        # know how big they'll end up being.
-        force_zip64 = bool(iterable)
+        # We need to force using zip64 extensions for unsized iterables since
+        # we don't know how big they'll end up being.
+        force_zip64 = bool(iterable) and size is None
 
         # Convert paths and data into iterables
         if path:
@@ -941,8 +1111,7 @@ class ZipStream:
         # Write central directory file headers
         centDirOffset = self._pos
         for zinfo in self._filelist:
-            for x in zinfo._central_directory_header_data():
-                yield self._track(x)
+            yield self._track(zinfo._central_directory_header_data())
 
         # Write end of central directory record
         zip64EndRecStart = self._pos
@@ -962,8 +1131,6 @@ class ZipStream:
                 centDirSize,
                 centDirOffset
             )
-            yield self._track(zip64EndRec)
-
             zip64LocRec = struct.pack(
                 structEndArchive64Locator,
                 stringEndArchive64Locator,
@@ -971,7 +1138,8 @@ class ZipStream:
                 zip64EndRecStart,
                 1
             )
-            yield self._track(zip64LocRec)
+            yield self._track(zip64EndRec + zip64LocRec)
+
             centDirCount = min(centDirCount, 0xFFFF)
             centDirSize = min(centDirSize, 0xFFFFFFFF)
             centDirOffset = min(centDirOffset, 0xFFFFFFFF)
@@ -986,15 +1154,10 @@ class ZipStream:
             centDirOffset,
             len(self._comment)
         )
-        yield self._track(endRec)
-        yield self._track(self._comment)
+        yield self._track(endRec + self._comment)
 
-    def _get_size(self):
-        """Calculate the final size of the zip stream as files are added"""
-        # The aim is to store as little data as possible and avoid recalculating
-        # the size of every file every time. By storing some data on how much
-        # space is required for the currently-counted files, we can just add to
-        # it for every new file.
+    def _add_size_from_file(self, arcname, size):
+        """Add the file to the calculated size of the ZipStream"""
 
         # Need to prevent multiple threads from reading _size_prog, calculating
         # independently, then all writing back conflicting progress.
@@ -1004,77 +1167,56 @@ class ZipStream:
             # exceeding a limit.
             (num_files, files_size, cdfh_size) = self._size_prog
 
-            while True:
-                try:
-                    arcname, size = self._to_count.popleft()
-                except IndexError:
-                    break
+            # Get the number of bytes the arcname uses by encoding it in
+            # the same way that ZipStreamInfo._encodeFilenameFlags does
+            try:
+                arcname_len = len(arcname.encode("ascii"))
+            except UnicodeEncodeError:
+                arcname_len = len(arcname.encode("utf-8"))
 
-                # Get the number of bytes the arcname uses by encoding it in
-                # the same way that ZipStreamInfo._encodeFilenameFlags does
-                try:
-                    arcname_len = len(arcname.encode("ascii"))
-                except UnicodeEncodeError:
-                    arcname_len = len(arcname.encode("utf-8"))
+            # Calculate if zip64 extensions are required in the same way that
+            # ZipStreamInfo.file_data does
+            uses_zip64 = size * ZIP64_ESTIMATE_FACTOR > ZIP64_LIMIT
 
-                # Calculate if zip64 extensions are required in the same way that
-                # ZipStreamInfo.file_data does
-                uses_zip64 = size * ZIP64_ESTIMATE_FACTOR > ZIP64_LIMIT
+            # Track the number of extra records in the central directory file
+            # header encoding this file will require
+            cdfh_extras = 0
 
-                # Track the number of extra records in the central directory file
-                # header encoding this file will require
-                cdfh_extras = 0
+            # Any files added after the size exceeds the zip64 limit will
+            # require an extra record to encode their location.
+            if files_size > ZIP64_LIMIT:
+                cdfh_extras += 1
 
-                # Any files added after the size exceeds the zip64 limit will
-                # require an extra record to encode their location.
-                if files_size > ZIP64_LIMIT:
-                    cdfh_extras += 1
+            # FileHeader
+            files_size += sizeFileHeader + arcname_len # 30 + name len
 
-                # FileHeader
-                files_size += sizeFileHeader + arcname_len # 30 + name len
+            # Folders don't have any data or require any extra records
+            if arcname[-1] != "/":
 
-                # Folders don't have any data or require any extra records
-                if arcname[-1] not in PATH_SEPARATORS:
+                # When using zip64, the size and compressed size of the file are
+                # written as an extra field in the FileHeader.
+                if uses_zip64:
+                    files_size += 20  # struct.calcsize('<HHQQ')
 
-                    # When using zip64, the size and compressed size of the file are
-                    # written as an extra field in the FileHeader.
-                    if uses_zip64:
-                        files_size += 20  # struct.calcsize('<HHQQ')
+                # file data
+                files_size += size
 
-                    # file data
-                    files_size += size
+                # DataDescriptor
+                files_size += 24 if uses_zip64 else 16  # struct.calcsize('<LLQQ' if zip64 else '<LLLL')
 
-                    # DataDescriptor
-                    files_size += 24 if uses_zip64 else 16  # struct.calcsize('<LLQQ' if zip64 else '<LLLL')
+                # Storing the size of a large file requires 2 extra records
+                # (size and compressed size)
+                if size > ZIP64_LIMIT:
+                    cdfh_extras += 2
 
-                    # Storing the size of a large file requires 2 extra records
-                    # (size and compressed size)
-                    if size > ZIP64_LIMIT:
-                        cdfh_extras += 2
+            cdfh_size += sizeCentralDir  # 46
+            cdfh_size += arcname_len
 
-                cdfh_size += sizeCentralDir  # 46
-                cdfh_size += arcname_len
+            # Add space for extra data
+            if cdfh_extras:
+                cdfh_size += 4 + (8 * cdfh_extras)  # struct.calcsize('<HH' + 'Q' * cdfh_extras)
 
-                # Add space for extra data
-                if cdfh_extras:
-                    cdfh_size += 4 + (8 * cdfh_extras)  # struct.calcsize('<HH' + 'Q' * cdfh_extras)
-
-                num_files += 1
+            num_files += 1
 
             # Record the current progress for next time
             self._size_prog = (num_files, files_size, cdfh_size)
-
-        # Calculate the amount of data the end of central directory needs. This
-        # is computed every time since it depends on the other metrics. Also,
-        # it means that we don't have to deal with detecting if the comment
-        # changes.
-        eocd_size = sizeEndCentDir + len(self._comment)  # 22 + comment len
-        if (
-            num_files > ZIP_FILECOUNT_LIMIT or
-            files_size > ZIP64_LIMIT or
-            cdfh_size > ZIP64_LIMIT
-        ):
-            eocd_size += sizeEndCentDir64  # 56
-            eocd_size += sizeEndCentDir64Locator  # 20
-
-        return cdfh_size + files_size + eocd_size
